@@ -2,23 +2,18 @@
 // Implementation of base text editor class for FLTK.
 //
 
+#include "../fltk/Fl_Editor.h"
+
 #if FL_MAJOR_VERSION == 1
-#	include <FL/Fl_Editor.H>
 #	include <FL/fl_ask.H>
 #	include <FL/Fl_Group.H>
 #	include <FL/fl_draw.H>
 #else
-#	include <fltk/Fl_Editor.h>
 #	include <fltk/fl_ask.h>
 #	include <fltk/Fl_Group.h>
 #	include <fltk/fl_draw.h>
 #endif
 
-// ugly workaround for drawing bug with FLTK 2.0 CVS													
-#if (FL_MAJOR_VERSION > 1)							
-#define FULL_REDRAW_ALWAYS 1
-#endif
-
 // **************************************************
 //						Callbacks
 // **************************************************
@@ -51,6 +46,7 @@ FL_API short Fl_EditorEngine::MeasureTextWidth(const char *t)
 	uchar csize = fl_size();
 #endif
 	short lwide = 0;
+	if(t && *t)
 	{
 		fl_font(this->textfont(), this->textsize());
 		lwide =  (short)fl_width(t);
@@ -101,11 +97,7 @@ FL_API Fl_Editor::Fl_Editor(int x, int y, int w, int h, const char *cap)
 	
 	color(7);			// default to white background
 	textcolor(FL_BLACK);
-#if (FL_MAJOR_VERSION > 1)
-	textfont(FL_TIMES);
-#else
 	textfont(FL_TIMES);		// default to 12-point Times font
-#endif
 	textsize(12);
 	setfont();
 	this->begin();
@@ -139,7 +131,11 @@ FL_API Fl_Editor::Fl_Editor(int x, int y, int w, int h, const char *cap)
 	engine->textsize(textsize());
 	
 	// add the scroller 
+#if FL_MAJOR_VERSION > 1
+	scroller = new Fl_Scrollbar(w-(SCROLLERWIDTH + bdx), bdy, SCROLLERWIDTH, (h - bdy));
+#else
 	scroller = new Fl_Scrollbar((x + w)-(SCROLLERWIDTH + bdx), y+bdy, SCROLLERWIDTH, (h - bdy));
+#endif
 	scroller->callback((Fl_Callback *)cb_EditScroller);
 	scroller->user_data(this);
 	scroller->value(StartLine, 10, 0, engine->GetLineCount() * 10);
@@ -189,6 +185,11 @@ FL_API short Fl_Editor::CursorX()
 	wString curline = engine->GetLine(engine->CursorRow());
 	curline.ChopAt(engine->CursorColumn());
 	left = editbox->x() + LEFTMARGIN + (short)fl_width(curline.Get()); 
+#if FL_MAJOR_VERSION > 1
+	left -= x();
+#endif
+	if(left < LEFTMARGIN)
+		left = LEFTMARGIN;
 	return(left);
 }
 
@@ -197,6 +198,11 @@ FL_API short Fl_Editor::CursorY()
 	long top;
 // TODO: Update this for lines with variable height
 	top = editbox->y() + TOPMARGIN + ((engine->MeasureTextHeight("X")) * (engine->CursorRow() - StartLine)); 
+#if FL_MAJOR_VERSION > 1
+	top -= y();
+#endif
+	if(top < TOPMARGIN)
+		top = TOPMARGIN;
 	return((short)top);
 }
 
@@ -212,7 +218,12 @@ FL_API void Fl_Editor::HandleScroll()
 	if ( oldstart != newstart )
 	{
 		this->window()->make_current();
-		draw_clip(this, editbox->x(), editbox->y(), editbox->w(), editbox->h());
+		SyncDisplay();
+#if FL_MAJOR_VERSION > 1
+//		draw_clip(this, x() + editbox->x(), y() + editbox->y(), editbox->w(), editbox->h());
+#else
+//		draw_clip(this, editbox->x(), editbox->y(), editbox->w(), editbox->h());
+#endif
 	}
 }
 
@@ -265,47 +276,16 @@ FL_API void Fl_Editor::ShowCursor(CursorStyles show)
 	scroller->value(SafeStart, SafeSize * 10, 0, SafeTotal * 10);
 }
 
-#if (FL_MAJOR_VERSION > 1)
-#define FULL_REDRAW_ALWAYS 1
-#endif
 
 FL_API void Fl_Editor::draw()
 {
 	ShowCursor(CURSOR_HIDE);
-#ifdef FULL_REDRAW_ALWAYS
-		// total redraw
-		// draw base widget outline and label, if any
-#if (FL_MAJOR_VERSION > 1)
-        Fl_Group::draw();
-//		draw_box(x(), y(), w(), h(), flags());
-//		draw_label(x(), y(), w(), h(), flags());
-#else
-		draw_box();
-		draw_label();
-#endif		
-        // draw text on background
-		int X,Y,W,H;
-		setfont();
-		fl_clip_box(editbox->x(), editbox->y(), editbox->w(), editbox->h(), X, Y, W, H);
-		draw_clip(this, X, Y, W, H);
-		// now draw all the children atop the background:
-#if (FL_MAJOR_VERSION > 1)
-//		for (int i = children(); i > 0; i--)
-//			draw_child(*child(i));
-#else
-		Fl_Widget *const*a = array();
-		for (int i = children(); i --; a ++)
-			draw_child(**a);
-#endif
-#else
 	if(damage() & FL_DAMAGE_ALL)
 	{
 		// total redraw
 		// draw base widget outline and label, if any
 #if (FL_MAJOR_VERSION > 1)
         Fl_Group::draw();
-//		draw_box(x(), y(), w(), h(), flags());
-//		draw_label(x(), y(), w(), h(), flags());
 #else
 		draw_box();
 		draw_label();
@@ -327,8 +307,8 @@ FL_API void Fl_Editor::draw()
 	}
 	else
 	{
-		if (damage() & DAMAGE_EDITOR)
-		{
+		if (DamageFlags & DAMAGE_EDITOR)
+    	{
 			//  redraw editing area only.
 			int X,Y,W,H;
 			setfont();
@@ -337,7 +317,7 @@ FL_API void Fl_Editor::draw()
 			// update scroll bar 
 			draw_child(*scroller);
 		}
-		else if((damage() & DAMAGE_LINE) || (damage() & DAMAGE_PARTIAL))
+		else if((DamageFlags & DAMAGE_LINE) || (DamageFlags & DAMAGE_PARTIAL))
 		{
 			EditPosition curpos = engine->GetPosition();
 			EditPosition originalpos = DrawFrom;
@@ -352,7 +332,11 @@ FL_API void Fl_Editor::draw()
 			{
 				wString current = engine->GetLine(curline);
 				if(curline >= DrawFrom.Row && curline <= curpos.Row)
+#if FL_MAJOR_VERSION > 1
+					DrawLine(curline, drawx - x(), (editbox->y() - y()) + drawy, current, *DrawState);
+#else
 					DrawLine(curline, drawx, editbox->y() + drawy, current, *DrawState);
+#endif
 				drawy += engine->MeasureTextHeight(current);
 			}
 			DrawFrom = originalpos;
@@ -369,7 +353,7 @@ FL_API void Fl_Editor::draw()
 #endif
 		}
 	}
-#endif // FULL_REDRAW_ALWAYS
+	DamageFlags = 0;
 	ShowCursor(CURSOR_UNHIDE);
 	draw_child(*Cursor);
 }
@@ -386,6 +370,10 @@ FL_API void Fl_Editor::draw_clip(void *v, int X, int Y, int W, int H)
 	top = S->editbox->y() + TOPMARGIN;
 	wide = S->editbox->w() - LEFTMARGIN;
 	high = S->editbox->h() - TOPMARGIN;
+#if FL_MAJOR_VERSION > 1
+	left -= S->x();
+	top -= S->y();
+#endif
 	lineheight = S->engine->MeasureTextHeight("X");
 	S->VisibleLines = high / lineheight; 
 
@@ -487,6 +475,12 @@ FL_API int Fl_Editor::handle(int event)
 			handled = 1;
 			break;
 		case FL_UNFOCUS:
+			if(Fl::focus() == this)
+			{
+				// workaround for strange focus behaviour when clicking the scrollbar, then the text area
+				handle(FL_FOCUS);
+				break;
+			}
 #ifndef WIN32
 			if ( marking )
 				CopySelection(false);	// set selection() so normal X mark-then-paste works.
@@ -494,7 +488,7 @@ FL_API int Fl_Editor::handle(int event)
 			if(!HideCursor())
 			{
 				HideCursor(myHideCursor = true);
-				damage(DAMAGE_PARTIAL);
+				damageit(DAMAGE_PARTIAL);
 			}
 			handled = 1;
 			break;
@@ -531,7 +525,7 @@ FL_API int Fl_Editor::handle(int event)
 				{
 					Fl::paste(*this);
 					//if (Fl::focus()==this)  handled = 1;	//remove for Motif behavior
-					damage(DAMAGE_EDITOR);
+					damageit(DAMAGE_EDITOR);
 				}
 #endif
 			}
@@ -543,7 +537,7 @@ FL_API int Fl_Editor::handle(int event)
 				{
 					engine->Command(CLEAR_MARKS);
 					selectlines = marking = false;
-					damage(DAMAGE_EDITOR);
+					damageit(DAMAGE_EDITOR);
 				}
 				switch ( Fl::event_clicks() % 3 )  // clicking four, five or six times is the same as one, two, or three...
 				{
@@ -559,7 +553,7 @@ FL_API int Fl_Editor::handle(int event)
 						if ( engine->CursorRow() == Mark.Row )
 							engine->Command(MOVE_EOL);
 						engine->Command(MARK_END);
-						damage(DAMAGE_PARTIAL);
+						damageit(DAMAGE_PARTIAL);
 						break;
 					case 1:		// double click
 						handled = 1;
@@ -576,7 +570,7 @@ FL_API int Fl_Editor::handle(int event)
 								engine->Command(MOVE_LEFT);
 							engine->Command(MARK_END);
 						}
-						damage(DAMAGE_PARTIAL);
+						damageit(DAMAGE_PARTIAL);
 						break;
 					case 0:		// single click
 						handled = 1;
@@ -589,7 +583,8 @@ FL_API int Fl_Editor::handle(int event)
 						}
 						if(Browsemode)
 							engine->Command(MOVE_SOL);
-						damage(DAMAGE_PARTIAL);	// force repaint of cursor
+						// force repaint of cursor
+						damageit(DAMAGE_PARTIAL);
 						//ShowCursor(CURSOR_UNHIDE);
 						break;
 				}
@@ -648,7 +643,7 @@ FL_API int Fl_Editor::handle(int event)
 						engine->Command(MOVE_DOWN);
 						TimerEnable = true;
 						Fl::add_timeout(1.0/(float)_scrollspeed, dragtimer, (void *)this);
-						damage(DAMAGE_EDITOR);
+						damageit(DAMAGE_EDITOR);
 					}
 					else if ( (Fl::event_y() < y()) && StartLine )
 					{
@@ -656,14 +651,14 @@ FL_API int Fl_Editor::handle(int event)
 						engine->Command(MOVE_UP);
 						TimerEnable = true;
 						Fl::add_timeout(1.0/(float)_scrollspeed, dragtimer, (void *)this);
-						damage(DAMAGE_EDITOR);
+						damageit(DAMAGE_EDITOR);
 					}
 					else
 						TimerEnable = false;
 					engine->Command(MARK_END);
 				}
 //				ShowCursor(CURSOR_UNHIDE);
-				damage(DAMAGE_PARTIAL);
+				damageit(DAMAGE_PARTIAL);
 				handled = 1;
 			}
 			break;
@@ -674,7 +669,7 @@ FL_API int Fl_Editor::handle(int event)
 				HandleScroll();
 				break;
 			}
-			damage(DAMAGE_PARTIAL);
+			damageit(DAMAGE_PARTIAL);
 			handled = 1;
 #ifndef WIN32
 			CopySelection(false);	// set selection() so normal X mark-then-paste works.
@@ -696,14 +691,14 @@ FL_API int Fl_Editor::handle(int event)
 				// and move the cursor to the end of what was just inserted
 				engine->Command(MOVE_RIGHT, Fl::event_length());
 			}
-			damage(DAMAGE_EDITOR);
+			damageit(DAMAGE_EDITOR);
 			handled = 1;
 			break;
 	}  
 	if(!handled)
 		handled = Fl_Group::handle(event);
 	if ( StartLine != curstart )   // did we scroll during this event?
-		damage(DAMAGE_EDITOR);
+		damageit(DAMAGE_EDITOR);
 	return(handled);
 }
 
@@ -738,7 +733,7 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 					break;
 			}
 		if(handled)
-			damage(DAMAGE_EDITOR);
+			damageit(DAMAGE_EDITOR);
 		if ( ( !handled ) && !(state & (FL_CTRL|FL_ALT)) )
 		{
 			if ( marking )
@@ -748,7 +743,7 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 					engine->Command(DELETE_REGION);
 				engine->Command(CLEAR_MARKS);
 				engine->MoveTo(Mark.Row, Mark.Column);
-				damage(DAMAGE_EDITOR);
+				damageit(DAMAGE_EDITOR);
 			}
 			if(!Readonly)
 			{
@@ -762,14 +757,21 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 				else
 					engine->InsertCharacter(text[0]);
 			}
-			damage(DAMAGE_LINE);
+			damageit(DAMAGE_LINE);
 			handled = 1;
 		}
 	}
+#if FL_MAJOR_VERSION > 1
+	else if ( key >= FL_KP(0) && key <= FL_KP_Last )
+	{
+		// numeric keypad key
+		key -= FL_KP(0);
+#else
 	else if ( key >= FL_KP && key <= FL_KP_Last )
 	{
 		// numeric keypad key
 		key -= FL_KP;
+#endif
 		if ( !(state & FL_NUM_LOCK) )
 		{
 			// translate to cursor key and process
@@ -809,7 +811,7 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 					engine->Command(DELETE_CHARACTER);
 			engine->Command(INSERT_STRING, text);
 			engine->Command(MOVE_RIGHT, len);
-			damage(DAMAGE_LINE);
+			damageit(DAMAGE_LINE);
 			handled = 1;
 		}
 	}
@@ -825,7 +827,7 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 				unmark = marking;
 				if(!Readonly)
 					engine->InsertCharacter(HARD_RETURN);
-				damage(DAMAGE_PARTIAL);
+				damageit(DAMAGE_PARTIAL);
 				break;
 			case FL_BackSpace:
 				if(!Readonly)
@@ -833,12 +835,12 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 					{
 						engine->DeleteRegion();
 						engine->MoveTo(Mark.Row, Mark.Column);
-						damage(DAMAGE_EDITOR);
+						damageit(DAMAGE_EDITOR);
 					}
 					else
 					{
 						engine->BackSpaceCharacter();
-						damage(DAMAGE_LINE);
+						damageit(DAMAGE_LINE);
 					}
 				break;
 			case FL_Delete:
@@ -846,18 +848,18 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 					if ( state & FL_SHIFT )
 					{
 						Cut();
-						damage(DAMAGE_EDITOR);
+						damageit(DAMAGE_EDITOR);
 					}
 					else if ( marking )
 					{
 						engine->DeleteRegion();
 						engine->MoveTo(Mark.Row, Mark.Column);
-						damage(DAMAGE_EDITOR);
+						damageit(DAMAGE_EDITOR);
 					}
 					else
 					{
 						engine->DeleteCharacter();
-						damage(DAMAGE_LINE);
+						damageit(DAMAGE_LINE);
 					}
 				unmark = marking;
 				break;
@@ -868,7 +870,7 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 					engine->Command(CLEAR_MARKS);
 					if(!Readonly)
 						engine->InsertCharacter('\t');
-					damage(DAMAGE_LINE);
+					damageit(DAMAGE_LINE);
 				}
 				break;
 			case FL_Left:
@@ -886,13 +888,13 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 				if ( state & FL_CTRL )
 				{
 					Copy();
-					damage(DAMAGE_EDITOR);
+					damageit(DAMAGE_EDITOR);
 				}
 				else if ( state & FL_SHIFT )
 				{
 					if(!Readonly)
 						Paste();
-					damage(DAMAGE_EDITOR);
+					damageit(DAMAGE_EDITOR);
 				}
 				else
 					overstrike = !overstrike;
@@ -926,7 +928,7 @@ FL_API int Fl_Editor::HandleKey(int key, const char *text, int len, int state)
 	if(!iscursorkey)
 		if((starting_row != ending_row) || (starting_count != ending_count) || engine->wrapped())
 		{
-			damage(DAMAGE_PARTIAL);
+			damageit(DAMAGE_PARTIAL);
 			redraw();
 		}
 	return(handled);
@@ -971,9 +973,13 @@ FL_API void Fl_Editor::resize(int nx, int ny, int nw, int nh)
 	if ( !lineheight )
 		lineheight = 12;	// wild guess at line height if can't figure it out.
 	VisibleLines = editbox->h() / lineheight; 
+#if (FL_MAJOR_VERSION > 1)
+	nx -= x(); 
+	ny -= y();
+#endif
     scroller->resize((nx + nw) - (scroller->w() + bdx) + 1, ny + bdy, SCROLLERWIDTH, h() - (2*bdy));
     engine->ChangeWidth(EditWidth = editbox->w() - LEFTMARGIN);
-	damage(FL_DAMAGE_ALL);
+	damageit(FL_DAMAGE_ALL);
 	redraw();
 	ShowCursor(CURSOR_UNHIDE);
 }
@@ -1103,7 +1109,7 @@ FL_API int Fl_Editor::HandleCursorKey(int key, const char *text, int len, int st
 		StartLine = engine->CursorRow();
 	if ( engine->CursorRow() >= StartLine + VisibleLines )
 		StartLine = engine->CursorRow() - (VisibleLines - 1);
-	damage(DAMAGE_EDITOR);
+	damageit(DAMAGE_EDITOR);
 	return(handled);
 }
 
@@ -1116,7 +1122,7 @@ FL_API void Fl_Editor::Cut()
 	engine->Command(CLEAR_MARKS); 
 	engine->MoveTo(Mark.Row, Mark.Column);
 	marking = selectlines = 0; 
-	damage(DAMAGE_EDITOR);
+	damageit(DAMAGE_EDITOR);
 }
 
 
@@ -1257,7 +1263,7 @@ FL_API void Fl_Editor::SyncDisplay()
 			StartLine = engine->CursorRow();
 		if ( engine->CursorRow() >= StartLine + VisibleLines )
 			StartLine = engine->CursorRow() - (VisibleLines - 1);
-		damage(DAMAGE_EDITOR);
+		damageit(DAMAGE_EDITOR);
 }
 
 FL_API void Fl_Editor::browse(bool setit)
@@ -1265,5 +1271,5 @@ FL_API void Fl_Editor::browse(bool setit)
 	Browsemode = setit;  
 	Readonly = setit; 
 	HideCursor(setit); 
-	damage(DAMAGE_LINE); 
+	damageit(DAMAGE_LINE); 
 };
