@@ -2,8 +2,9 @@
 // Implementation of text editor class for FLTK.
 //
 
+#include "../fltk/Fl_FancyEditor.h"
+
 #if FL_MAJOR_VERSION == 1
-#	include <FL/Fl_FancyEditor.H>
 #	include <FL/fl_ask.H>
 #	include <FL/Fl_Input.H>
 #	include <FL/Fl_Check_Button.H>
@@ -12,7 +13,6 @@
 #	include <FL/Fl_Group.H>
 #	include <FL/fl_draw.H>
 #else
-#	include <fltk/Fl_FancyEditor.h>
 #	include <fltk/fl_ask.h>
 #	include <fltk/Fl_Input.h>
 #	include <fltk/Fl_Check_Button.h>
@@ -100,20 +100,13 @@ FL_API Fl_FancyEditor::Fl_FancyEditor(int x, int y, int w, int h, const char *ca
 	findopts.PromptReplace = true;
 	// add the status panel
 	short paneltop = (y+h) - PANELHEIGHT;
-//	Fl_Group *curgrp = Fl_Group::current();
-//	Fl_Group::current(0);
 	lights = new Fl_StatusPanel(x, paneltop, w, PANELHEIGHT);
-//	Fl_Group::current(curgrp);
-	this->add(lights);
 }
 
 FL_API Fl_FancyEditor::~Fl_FancyEditor()
 {
 	if ( lights )
-	{
-		this->remove(lights);
 		delete lights;
-	}
 }
 
 
@@ -382,7 +375,7 @@ FL_API Fl_Window *Fl_FancyEditor::MakeReplaceDialog()
 FL_API bool Fl_FancyEditor::FindNext()
 {
 	bool foundone = false;
-	damage(DAMAGE_PARTIAL);			// redraw line after search
+	damageit(DAMAGE_PARTIAL);			// redraw line after search
 	marking = selectlines = false;		// turn off marking for search
 	if ( findopts.SearchFor.Length() )
 	{
