@@ -4,16 +4,20 @@
 // and swap the engine and state in and out when switching buffers.
 //
 
+#include "../fltk/Fl_MultiEditor.h"
+
 #if FL_MAJOR_VERSION == 1
-#	include <FL/Fl_MultiEditor.H>
 #	include <FL/filename.H>
 #	include <FL/Fl_Menu.H>
 #	include <FL/fl_message.H>
+#   if FL_MINOR_VERSION == 0
+#   	define fl_filename_name filename_name
+#   endif
 #else
-#	include <fltk/Fl_MultiEditor.h>
 #	include <fltk/filename.h>
 #	include <fltk/Fl_Menu_.h>
 #	include <fltk/fl_message.h>
+#	define fl_filename_name filename_name
 #endif
 
 
@@ -75,7 +79,7 @@ void Fl_MultiEditor::Session(fledit_SessionInfo *sinf)
 		this->StartLine = sinf->StartLine;
 		readonly(sinf->ReadOnly);
 		CurrentSession = sinf;
-		damage(FL_DAMAGE_ALL);
+		damageit(FL_DAMAGE_ALL);
 		// don't know why, but it won't repaint until the next event, so make one
 		HandleKey(0, "", 0, FL_CTRL);
 		HandleScroll();
@@ -109,7 +113,7 @@ long Fl_MultiEditor::NewSession(const char *name)
 		sinf->Caption = new wString;
 		sinf->SessionName = new wString;
 		sinf->SessionNumber = ++SessionCount;
-		const char *temp = name ? filename_name(name) : NULL;
+		const char *temp = name ? fl_filename_name(name) : NULL;
 		if(temp)
             *sinf->SessionName = temp;
 		else
@@ -217,7 +221,7 @@ void Fl_MultiEditor::CloseCurrent()
 			CurrentSession->marking = marking = 0;
 			CurrentSession->StartLine = StartLine = 0;
 			*CurrentSession->FileName = "";
-			damage(FL_DAMAGE_ALL);
+			damageit(FL_DAMAGE_ALL);
 		}
 		if(CurrentSession != (fledit_SessionInfo*)target->UserData)
 		{
@@ -250,7 +254,7 @@ void Fl_MultiEditor::SaveCurrent(const char *fn)
 			fclose(out);
 		}
 		*CurrentSession->FileName = filename;
-		const char *temp = filename_name(fn);
+		const char *temp = fl_filename_name(fn);
 		if(!temp)
 			temp = fn;
 		if(temp)
