@@ -1,8 +1,4 @@
-#if FL_MAJOR_VERSION == 1
-#	include <FL/Fl_ProgressBox.H>
-#else
-#	include <fltk/Fl_ProgressBox.h>
-#endif
+#include "../fltk/Fl_ProgressBox.h"
 #include <stdio.h>
 
 FL_API Fl_ProgressBox::Fl_ProgressBox(int x, int y, int w, int h, const char *lbl)
@@ -36,7 +32,11 @@ FL_API void Fl_ProgressBox::draw()
 	if(mPresent < mMin)
 		mPresent = mMin;
 	pct = (mPresent - mMin) / mMax;
+#if (FL_MAJOR_VERSION > 1)
+	fl_rectf(bdx, bdy, (int)(((double)w() - 2*bdx) * pct), h() - 2*bdy);
+#else
 	fl_rectf(x() + bdx, y() + bdy, (int)(((double)w() - 2*bdx) * pct), h() - 2*bdy);
+#endif	
 	if(mShowPct)
 	{
 		char buffer[30];
@@ -44,10 +44,11 @@ FL_API void Fl_ProgressBox::draw()
 		fl_color(textcolor());
 #if (FL_MAJOR_VERSION > 1)
 		fl_font(this->label_font(), this->label_size());
+		fl_draw(buffer, (w() - fl_width(buffer))/2, fl_height() + (((h() - 2*bdy) - fl_height())/2));
 #else
 		fl_font(this->labelfont(), this->labelsize());
-#endif
 		fl_draw(buffer, x() + (w() - fl_width(buffer))/2, y() + fl_height() + (((h() - 2*bdy) - fl_height())/2));
+#endif
 	}
 }
 
