@@ -1,8 +1,4 @@
-#if FL_MAJOR_VERSION == 1
-#	include <FL/Fl_StatusPanel.H>
-#else
-#	include <fltk/Fl_StatusPanel.h>
-#endif
+#include "../fltk/Fl_StatusPanel.h"
 #include <stdio.h>
 
 
@@ -17,18 +13,19 @@ FL_API Fl_StatusPanel::Fl_StatusPanel(int nx, int ny, int nw, int nh, const char
 	bdy = box()->dy();
 	bdw = box()->dw();
 	bdh = box()->dh();
+	int ttop = bdy;
+	nx = bdx;
 #else
 	bdx = Fl::box_dx(box());
 	bdy = Fl::box_dy(box());
 	bdw = Fl::box_dw(box());
 	bdh = Fl::box_dh(box());
-#endif
 	int ttop = ny + bdy;
-	int theight = (nh - bdh);
 	nx += bdx;
+#endif
+	int theight = (nh - bdh);
 	nw -= bdx;
 	
-
 	StatusMsg = new Fl_Tile(nx, ttop, nw - (3*33), theight, "");
 	StatusMsg->box(FL_DOWN_BOX);
 #if (FL_MAJOR_VERSION > 1)
@@ -47,6 +44,8 @@ FL_API Fl_StatusPanel::Fl_StatusPanel(int nx, int ny, int nw, int nh, const char
 #endif
 
 	this->resizable(StatusMsg);
+	
+	StatusMsg->end();
 
 	INSlight = new Fl_Tile((nx+nw) - (3*33), ttop, 33, theight, "INS");
 	INSlight->box(FL_DOWN_BOX);
@@ -84,17 +83,15 @@ FL_API Fl_StatusPanel::Fl_StatusPanel(int nx, int ny, int nw, int nh, const char
 #endif
 	NUMlight->end();
 
-	
-	PBar = new Fl_ProgressBox(StatusMsg->x() + smbdx, StatusMsg->y() + smbdy, StatusMsg->w() / 2, StatusMsg->h() - smbdh, "");
 #if (FL_MAJOR_VERSION > 1)
+	PBar = new Fl_ProgressBox(StatusMsg->x() + smbdx, StatusMsg->y() + smbdy, StatusMsg->w() / 2, StatusMsg->h() - smbdh, "");
 	PBar->label_size(10);
 #else
+	PBar = new Fl_ProgressBox(StatusMsg->x() + smbdx, StatusMsg->y() + smbdy, StatusMsg->w() / 2, StatusMsg->h() - smbdh, "");
 	PBar->labelsize(10);
 #endif
 	PBar->hide();
 	
-	StatusMsg->end();
-	
 	this->end();
 }
 
@@ -130,44 +127,56 @@ FL_API Fl_StatusPanel::~Fl_StatusPanel()
 	}
 }
 
-FL_API int Fl_StatusPanel::updatestatus()
+// This won't update right away on turning *off* the cap/num lock, at least
+// on Linux, because the event for the keypress has the state still set,
+// so you don't get the state cleared until the *next* event.
+FL_API int Fl_StatusPanel::updatestatus(int event)
 {
 	// update the keyboard status lights
 	int changed = 1;
-	int key = Fl::event_key(); 
+	int key = 0;
 	int state = Fl::event_state();
-	switch ( key )
+	int capon = (CAPlight->label()[0] == 'C');
+	int numon = (NUMlight->label()[0] == 'N');
+		
+#if (FL_MAJOR_VERSION > 1) || (FL_MINOR_VERSION > 1)
+	if(event == FL_KEYUP)
+#endif
 	{
-		case FL_Caps_Lock:
-			// if pressing the key, toggle the state for the check below...
-			if(state & FL_CAPS_LOCK)
-				state &= ~FL_CAPS_LOCK;
-			else
-				state |= FL_CAPS_LOCK;
-			cap(state & FL_CAPS_LOCK);
-			break;
-		case FL_Num_Lock:
-			if(state & FL_NUM_LOCK)
-				state &= ~FL_NUM_LOCK;
-			else
-				state |= FL_NUM_LOCK;
-			num(state & FL_NUM_LOCK);
-			break;
-		default:
-			changed = 0;
-			cap(state & FL_CAPS_LOCK);
-			num(state & FL_NUM_LOCK);
-//			CAPlight->label(( state  & FL_CAPS_LOCK ) ? "CAP" : "");
-//			CAPlight->redraw();
-//			NUMlight->label(( state & FL_NUM_LOCK ) ? "NUM" : "");
-//			NUMlight->redraw();
-			break;
+		key = Fl::event_key();
+		if(key == FL_Insert)
+		{
+			INSlight->label((INSlight->label()[0] == 'I') ? "" : "INS");
+			redraw();
+		}
+	}
+	if(capon && !(state & FL_CAPS_LOCK))
+	{
+		cap(0);
+		redraw();
+	}
+	else if(!capon && (state & FL_CAPS_LOCK))
+	{
+		cap(1);
+		redraw();
+	}
+	if(numon  && !(state & FL_NUM_LOCK))
+	{
+		num(0);
+		redraw();
+	}
+	else if(!numon && (state & FL_NUM_LOCK))
+	{
+		num(1);
+		redraw();
 	}
 	return(changed);
 }
 
+
 FL_API int Fl_StatusPanel::handle(int event)
 {
-	return(updatestatus());
+	updatestatus(event);
+	return (Fl_Widget::handle(event));
 }
 
