@@ -567,12 +567,6 @@ rclError EditorEngine::LoadFrom(FILE *infile)
 				break;		// end of line, so add it.
 			}
 		}
-//			if((row == Cursor.Row) && CharCount) // if first row, copy back to same line
-//				{
-//					*Lines[Cursor.Row] = s;
-//					++row;						// bump to next row...
-//				}
-//			else
 		InsertLine(s,++row,aslast);		// otherwise insert new line
 		if (LastResult == NOMEMORY || LastResult == LIMITS_FULL_ERROR)
 		{
@@ -1196,7 +1190,7 @@ rclError EditorEngine::InsertLine(const wString& str, long atline, bool aslast)
 
 short EditorEngine::MeasureTextWidth(const char *t)
 {
-	return(strlen(t));	   // default is width in characters
+	return((t && *t) ? strlen(t) : 0);	   // default is width in characters
 }
 
 short EditorEngine::MeasureTextHeight(const char *t)
