@@ -68,7 +68,7 @@ RCLDLL wString::wString(const char *what, long  where, long  howmuch, bool strip
 				Strip();
 			else
 				{
-					*this += wString(howmuch, ' ');
+					*this += wString(' ', howmuch);
 					this->ChopAt(howmuch);
 				}
 		}
