@@ -5,7 +5,8 @@
 #else
 #	include <fltk/Fl.h>
 #	include <fltk/Fl_Window.h>
-#	include <fltk/Fl_ProgressBox.h>
+#	include <fltk/Fl_ProgressBar.h>
+#	define Fl_ProgressBox Fl_ProgressBar
 #endif
 
   Fl_Window* w;
